/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Vidur Apparao <vidur@netscape.com> (original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

SCHEMA_ATOM(sAnyType_atom, "anyType")
SCHEMA_ATOM(sString_atom, "string")
SCHEMA_ATOM(sNormalizedString_atom, "normalizedString")
SCHEMA_ATOM(sToken_atom, "token")
SCHEMA_ATOM(sByte_atom, "byte")
SCHEMA_ATOM(sUnsignedByte_atom, "unsignedByte")
SCHEMA_ATOM(sBase64Binary_atom, "base64Binary")
SCHEMA_ATOM(sHexBinary_atom, "hexBinary")
SCHEMA_ATOM(sInteger_atom, "integer")
SCHEMA_ATOM(sPositiveInteger_atom, "positiveInteger")
SCHEMA_ATOM(sNegativeInteger_atom, "negativeInteger")
SCHEMA_ATOM(sNonnegativeInteger_atom, "nonNegativeInteger")
SCHEMA_ATOM(sNonpositiveInteger_atom, "nonPositiveInteger")
SCHEMA_ATOM(sInt_atom, "int")
SCHEMA_ATOM(sUnsignedInt_atom, "unsignedInt")
SCHEMA_ATOM(sLong_atom, "long")
SCHEMA_ATOM(sUnsignedLong_atom, "unsignedLong")
SCHEMA_ATOM(sShort_atom, "short")
SCHEMA_ATOM(sUnsignedShort_atom, "unsignedShort")
SCHEMA_ATOM(sDecimal_atom, "decimal")
SCHEMA_ATOM(sFloat_atom, "float")
SCHEMA_ATOM(sDouble_atom, "double")
SCHEMA_ATOM(sBoolean_atom, "boolean")
SCHEMA_ATOM(sTime_atom, "time")
SCHEMA_ATOM(sDateTime_atom, "dateTime")
SCHEMA_ATOM(sDuration_atom, "duration")
SCHEMA_ATOM(sDate_atom, "date")
SCHEMA_ATOM(sGMonth_atom, "gMonth")
SCHEMA_ATOM(sGYear_atom, "gYear")
SCHEMA_ATOM(sGYearMonth_atom, "gYearMonth")
SCHEMA_ATOM(sGDay_atom, "gDay")
SCHEMA_ATOM(sGMonthDay_atom, "gMonthDay")
SCHEMA_ATOM(sName_atom, "name")
SCHEMA_ATOM(sQName_atom, "QName")
SCHEMA_ATOM(sNCName_atom, "NCName")
SCHEMA_ATOM(sAnyURI_atom, "anyURI")
SCHEMA_ATOM(sLanguage_atom, "language")
SCHEMA_ATOM(sID_atom, "ID")
SCHEMA_ATOM(sIDREF_atom, "IDREF")
SCHEMA_ATOM(sIDREFS_atom, "IDREFS")
SCHEMA_ATOM(sENTITY_atom, "ENTITY")
SCHEMA_ATOM(sENTITIES_atom, "ENTITIES")
SCHEMA_ATOM(sNOTATION_atom, "NOTATION")
SCHEMA_ATOM(sNMTOKEN_atom, "NMTOKEN")
SCHEMA_ATOM(sNMTOKENS_atom, "NMTOKENS")

SCHEMA_ATOM(sElement_atom, "element")
SCHEMA_ATOM(sModelGroup_atom, "group")
SCHEMA_ATOM(sAny_atom, "any")
SCHEMA_ATOM(sAttribute_atom, "attribute")
SCHEMA_ATOM(sAttributeGroup_atom, "attributeGroup")
SCHEMA_ATOM(sSimpleType_atom, "simpleType")
SCHEMA_ATOM(sComplexType_atom, "complexType")
SCHEMA_ATOM(sSimpleContent_atom, "simpleContent")
SCHEMA_ATOM(sComplexContent_atom, "complexContent")
SCHEMA_ATOM(sAll_atom, "all")
SCHEMA_ATOM(sChoice_atom, "choice")
SCHEMA_ATOM(sSequence_atom, "sequence")
SCHEMA_ATOM(sAnyAttribute_atom, "anyAttribute")
SCHEMA_ATOM(sRestriction_atom, "restriction")
SCHEMA_ATOM(sExtension_atom, "extension")
SCHEMA_ATOM(sAnnotation_atom, "annotation")
SCHEMA_ATOM(sList_atom, "list")
SCHEMA_ATOM(sUnion_atom, "union")

SCHEMA_ATOM(sMinExclusive_atom, "minExclusive")
SCHEMA_ATOM(sMinInclusive_atom, "minInclusive")
SCHEMA_ATOM(sMaxExclusive_atom, "maxExclusive")
SCHEMA_ATOM(sMaxInclusive_atom, "maxInclusive")
SCHEMA_ATOM(sTotalDigits_atom, "totalDigits")
SCHEMA_ATOM(sFractionDigits_atom, "fractionDigits")
SCHEMA_ATOM(sLength_atom, "length")
SCHEMA_ATOM(sMinLength_atom, "minLength")
SCHEMA_ATOM(sMaxLength_atom, "maxLength")
SCHEMA_ATOM(sEnumeration_atom, "enumeration")
SCHEMA_ATOM(sWhiteSpace_atom, "whiteSpace")
SCHEMA_ATOM(sPattern_atom, "pattern")
