//Microsoft Developer Studio generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "winresrc.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE
BEGIN
    "#include ""winresrc.h""\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_PLUGIN_DOWNLOAD DIALOG DISCARDABLE  0, 0, 225, 165
STYLE DS_MODALFRAME | WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU
CAPTION "Title"
FONT 8, "MS Sans Serif"
BEGIN
    LTEXT           "Information on this page requires a plug-in for:",
                    IDC_STATIC_INFO,7,7,211,10
    LTEXT           "Communicator can retrieve the plug-in for you from:",
                    IDC_STATIC_LOCATION,7,32,211,8
    DEFPUSHBUTTON   "Get the Plug-in",IDC_GET_PLUGIN,41,144,64,14
    PUSHBUTTON      "Cancel",IDC_BUTTON_CANCEL,109,144,64,14
    CTEXT           "type/x-type",IDC_STATIC_INFOTYPE,7,18,211,8
    LTEXT           "The SmartUpdate feature makes it easy to install new plug-ins. To take advantage of SmartUpdate, you must enable Java, JavaScript and AutoInstall in the Advanced panel of the Preferences, then click the plug-in icon on the page.",
                    IDC_STATIC_WARNING,7,74,211,61
    LTEXT           "Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static Static ",
                    IDC_STATIC_URL,7,43,211,24
END


#ifndef _MAC
/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,0,15
 PRODUCTVERSION 1,0,0,15
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
            VALUE "Comments", "\0"
            VALUE "CompanyName", "mozilla.org\0"
            VALUE "FileDescription", "Default Plug-in\0"
            VALUE "FileExtents", "*\0"
            VALUE "FileOpenName", "Mozilla Default Plug-in (*.*)\0"
            VALUE "FileVersion", "1, 0, 0, 15\0"
            VALUE "InternalName", "DEFPLUGIN\0"
            VALUE "LegalCopyright", "Copyright © 1995-2000\0"
            VALUE "LegalTrademarks", "\0"
            VALUE "MIMEType", "*\0"
            VALUE "OriginalFilename", "NPNUL32.DLL\0"
            VALUE "PrivateBuild", "\0"
            VALUE "ProductName", "Mozilla Default Plug-in\0"
            VALUE "ProductVersion", "1, 0, 0, 15\0"
            VALUE "SpecialBuild", "\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END

#endif    // !_MAC


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_PLUGICON            ICON    DISCARDABLE     "plugicon.ico"

/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE
BEGIN
    IDD_PLUGIN_DOWNLOAD, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 218
        TOPMARGIN, 7
        BOTTOMMARGIN, 158
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE
BEGIN
    IDS_INFO                "Information on this page requires a plug-in for:"
END

STRINGTABLE DISCARDABLE
BEGIN
    IDS_LOCATION            "You can retrieve the plug-in from:"
    IDS_TITLE               "Plug-in Not Loaded"
    IDS_QUESTION            "What would you like to do?"
    IDS_WARNING_JS          "To retrieve the plug-in now, click the Get the Plug-in button and then select the plug-in you need from the page that appears."
    IDS_WARNING_OFFLINE     "However, you are currently offline. If you would like to get the plug-in, click Cancel, select ""Work Online"" from the File menu, then click the plug-in icon on the page."
    IDS_FINDER_PAGE         "Plug-in Finder page"
    IDS_CLICK_TO_GET        "Click here to get the plugin"
    IDS_CLICK_WHEN_DONE     "After installing the plugin, click here."
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

