@import url("chrome://calendar/content/datetimepickers/datepicker.css");
@import url("chrome://calendar/content/datetimepickers/timepicker.css");

datetimepicker {
  -moz-binding: url("chrome://calendar/content/datetimepickers/datetimepicker.xbl#datetimepicker");
}

.menu-right
{
  visibility : hidden;
}

